--[[
	NOTE: Many parts of this Addon are copied from RCLootCouncil. Credit to the original author(s). 
]]--

SFLoot = LibStub("AceAddon-3.0"):NewAddon("SFLoot", "AceConsole-3.0", "AceEvent-3.0", "AceComm-3.0", "AceSerializer-3.0", "AceTimer-3.0");
local lwin = LibStub("LibWindow-1.1");
local db;

local frames = {};
frames.itemButtons = {};
local menuFrame;
local lootTable = {};
local currentItem = 1;
local responses = {
	[1] = "Pass",
	[2] = "Transmog",
	[3] = "Off",
	[4] = "Main",
};

local today = date("%Y/%m/%d", time())

local ROW_HEIGHT = 20;
local NUM_ROWS = 10;

function SFLoot:OnInitialize()

	self.scrollCols = {
		{ name = "",																	width = 20, },	-- 1 Class
		{ name = "Name",																	width = 150,},	-- 2 Candidate Name
		{ name = "Verwendung",						comparesort = ResponseSort, sortnext = 9,		width = 80,},	-- 3 Response
		{ name = "ilvl",															width = 45, },	-- 4 Total ilvl
		{ name = "Diff",																			width = 40, },	-- 5 ilvl difference
		{ name = "g1",			align = "CENTER",								width = 20, },	-- 6 Current gear 1
		{ name = "g2",			align = "CENTER",								width = 20, },	-- 7 Current gear 2
		{ name = "Items",		align = "CENTER",																	width = 40, },	-- 8 ilvl difference
		{ name = "Roll",		align = "CENTER", 	comparesort = RollSort,				width = 30, },	-- 9 Roll
	}

	self.db = LibStub("AceDB-3.0"):New("SFLootDB", defaults)
	
	self.defaults = {
		global = {
			localizedSubTypes = {},
		},
		profile = {
			ambiguate = false,
			UI = { -- stores all ui information
				['**'] = { -- Defaults
					y		= 0,
					x		= 0,
					point	= "CENTER",
					scale	= 0.8,
					bgColor = {0.25, 0.25, 0.25, 1},
					borderColor = {1, 1, 1, 0.2},
					border = "Interface\\Tooltips\\UI-Tooltip-Border",
					background = "Interface\\Tooltips\\UI-Tooltip-Background",
				},
				default = {}, -- base line
			},
			lootDB = {},
			-- List of items to ignore:
			ignore = {
				109693, -- Draenic Dust
				115502, -- Small Luminous Shard
				111245, -- Luminous Shard
				115504, -- Fractured Temporal Crystal
				113588, -- Temporal Crystal
				124442, -- Chaos Crystal (Legion)
				124441, -- Leylight Shard (Legion)
				141303,141304,141305, -- Essence of Clarity (Emerald Nightmare quest item)
				139390, --Artifact Research Note
			},
		},
	} -- defaults end	
	
	menuFrame = CreateFrame("Frame", "SFLoot_Frame_RightclickMenu", UIParent, "Lib_UIDropDownMenuTemplate");
	Lib_UIDropDownMenu_Initialize(menuFrame, self.RightClickMenu, "MENU")
	
	self.db = LibStub("AceDB-3.0"):New("SFLootDB", self.defaults, true)
	db = self.db.profile;
	
	self:LocalizeSubTypes()
end

function SFLoot:OnEnable()
	self:RegisterEvent("LOOT_OPENED", "OnEvent")
	self:RegisterEvent("LOOT_CLOSED", "OnEvent")
	self:RegisterEvent("PARTY_LOOT_METHOD_CHANGED", "OnEvent")
	self:RegisterEvent("RAID_INSTANCE_WELCOME", "OnEvent")
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "OnEvent")
	
	self:RegisterComm("SFLoot");
	
	self.realmName = select(2, UnitFullName("player"))
	self.playerName = self:UnitName("player")
	self.playerClass = select(2, UnitClass("player"))
	
	self.isMasterLooter, self.masterLooter = self:GetML();
	
	self.running = false;
	self.enabled = false;
	self.lootOpen = false;
	
	self.debug = true;
	
	self.version = GetAddOnMetadata("SFLoot", "Version");
	self.newVersion = false;
end

function SFLoot:OnEvent(event, ...)
	if event == "LOOT_OPENED" then 
		self.lootOpen = true
		self:LootOpened()
	elseif event == "LOOT_CLOSED" then
		self.lootOpen = false
	elseif event == "PARTY_LOOT_METHOD_CHANGED" then
		self:NewMLCheck()
	elseif event == "RAID_INSTANCE_WELCOME" then
		self:ScheduleTimer("OnRaidEnter", 2)	
	elseif event == "PLAYER_ENTERING_WORLD" then
		if IsInGuild("player") then
			self:SendCommand("guild", "version", self.version)
			if InGuildParty("player") then
				self:SendCommand("group", "request_loot")
			end
		end
	end
end

function SFLoot:Test()
	lootTable = {}

	self:AddItem("\124cffa335ee\124Hitem:142519::::::::110::::1:3467:\124h[Favor of the Prime Designate]\124h\124r", 1)
	self:AddItem("\124cffa335ee\124Hitem:142425::::::::110::::1:3467:\124h[Goldrune Legplates]\124h\124r", 2)
	self:AddItem("\124cffa335ee\124Hitem:142520::::::::110::::1:3467:\124h[Ring of Ascended Glory]\124h\124r", 3)
	self:AddItem("\124cffa335ee\124Hitem:142508::::::::110::::1:3467:\124h[Chains of the Valorous]\124h\124r", 4)
	self:AddItem("\124cffa335ee\124Hitem:142540::::::::110::::1:3467:\124h[Mantle of the Victorious Dead]\124h\124r", 5)
	self:AddItem("\124cffa335ee\124Hitem:142512::::::::110::::1:3467:\124h[Accursed Cuspid]\124h\124r", 6)
		
	self:MainFrameShow()
end

function SFLoot:LootOpened()
	if self.isMasterLooter and GetNumLootItems() > 0 then
		local updatedLootSlot = {}

		if not self.running then 
			lootTable = {};
		end
		
		for i = 1, GetNumLootItems() do
			local item = GetLootSlotLink(i)
			-- We have reopened the loot frame, so check if we should update .lootSlot
			if self.running then
				for session = 1, #lootTable do
					-- Just skip if we've already awarded the item or found a fitting lootSlot
					if not lootTable[session].awarded and not updatedLootSlot[session] then
						if item == lootTable[session].link then
							if i ~= lootTable[session].lootSlot then -- It has changed!
								lootTable[session].lootSlot = i -- and update it
								updatedLootSlot[session] = true
							end
							break
						end
					end
				end
			else
				local _, _, quantity, quality = GetLootSlotInfo(i)
				local itemid = self:GetItemIDFromLink(item);	
				
				if item and not db.ignore[itemid] and quantity > 0 and quality > 3 then 
					self:AddItem(item, i)
					self:Debug("added")
					self:Debug(#lootTable)
				elseif quantity == 0 then -- it's coin, just loot it
					LootSlot(i)
				end
			end
		end
		self:Debug("self.running = "..tostring(self.running));
		if #lootTable > 0 and not self.running then
			self:Debug("session started")
			self:StartSession()
		end
	end
end

function SFLoot:AddItem(item, index)
	local name, link, rarity, ilvl, iMinLevel, type, subType, iStackCount, equipLoc, texture = GetItemInfo(item)
	table.insert(lootTable, {
		["index"]       = #lootTable + 1,
		["lootSlot"]	= index,
		["awarded"]		= nil,
		["name"]		= name,
		["link"]		= link,
		["quality"]		= rarity,
		["ilvl"]		= ilvl,
		["subType"]		= subType,
		["equipLoc"]	= equipLoc,
		["texture"]		= texture,
		["boe"]			= SFLoot:IsItemBoE(link),
		["candidates"]  = {},
	})
	self:Debug("AddItem: "..tostring(index)..", ", name)
	-- Item isn't properly loaded, so update the data in 1 sec (Should only happen with /rc test)
	if not name then
		self:ScheduleTimer("AddItem", 1, item, index or #lootTable)
		self:Debug("timer")
	end
end

function SFLoot:StartSession()
	self.running = true
	self:MainFrameShow()
	self:SendCommand("group", "lootTable", lootTable)
end

function SFLoot:EndSession()
	self:SendCommand("group", "session_end")
	self.running = false
	self:CancelAllTimers()
end

function SFLoot:LootRoll(name, response)
	if not currentItem then return end
	if #lootTable == 0 then return end

	local roll = {};
	local item = lootTable[currentItem];
	local g1, g2;
	
	if item.equipLoc == "" and item.subType == self.db.global.localizedSubTypes["Artifact Relic"] then
		g1, g2 = self:GetArtifactRelics(item.link)
	else
	 	g1, g2 = self:GetPlayersGear(item.link, item.equipLoc)
	end
	local diff = 0
	if g2 then
		local g1diff, g2diff = select(4, GetItemInfo(g1)), select(4, GetItemInfo(g2))
		diff = g1diff >= g2diff and item.ilvl - g2diff or item.ilvl - g1diff
	elseif g1 then -- Artifact Relic might be nil
		diff = (item.ilvl - select(4, GetItemInfo(g1)))
	end	
	
	roll.player = name;
	roll.class = self.playerClass;
	roll.index = item.index;
	roll.lootSlot = item.lootSlot;
	roll.response = response;
	roll.ilvl = select(2, GetAverageItemLevel());
	roll.gear1 = g1;
	roll.gear2 = g2;
	roll.diff = diff;
	roll.value = math.random(100);

	self:SendCommand("group", "lootRoll", roll)
end

function SFLoot:HandleLootRoll(roll)
	local t = lootTable[roll.index];
	
	if t.candidates[roll.player] and roll.response > 1 then 
		return
	elseif t.candidates[roll.player] and roll.response == 1 then
		t.candidates[roll.player].response = roll.response;
	elseif not t.candidates[roll.player] then		
		t.candidates[roll.player] = {
			class = roll.class,
			response = roll.response,
			ilvl = roll.ilvl,
			diff = roll.diff,
			gear1 = roll.gear1,
			gear2 = roll.gear2,
			roll = roll.value,
		}
	end
	
	if roll.index == currentItem then
		self:SwitchItem(currentItem);
	end
end

function SFLoot:AwardItem(winner, session)
	if not lootTable[session].lootSlot then
		self:Debug("Session "..session.." didn't have lootSlot")
		return
	end

	if winner then
		local awarded = false
		if not self.lootOpen then -- we can't give out loot without the loot window open
			self:Print("Unable to give out loot without the loot window open.")
			--addon:Print(L["Alternatively, flag the loot as award later."])
			return
		end
		for i = 1, MAX_RAID_MEMBERS do
			self:Debug(GetMasterLootCandidate(lootTable[session].lootSlot, i))
			if self:UnitIsUnit(GetMasterLootCandidate(lootTable[session].lootSlot, i), winner) then
				GiveMasterLoot(lootTable[session].lootSlot, i)
				lootTable[session].awarded = winner;
				awarded = true
			end
		end

		if awarded then
			self:SendCommand("group", "awarded", winner, session)
			if self:HaveAllItemsBeenAwarded() then self:EndSession() end
		else 
			self:Print(format("Unable to give %s to %s - (player offline, left group or instance?)", lootTable[session].link, winner))
		end
	end
end

function SFLoot:HaveAllItemsBeenAwarded()
	local moreItems = true
	for i = 1, #lootTable do
		if not lootTable[i].awarded then
			moreItems = false
		end
	end
	return moreItems
end

function SFLoot:ItemAwarded(name, index)
	local item = lootTable[index]
		
	item.awarded = SFLoot.Ambiguate(name)
	
	self:UpdateItemDB(name, item)
	
	if index == currentItem then
		self:SwitchItem(currentItem)
	end
end

function SFLoot:UpdateItemDB(name, item)
	today = date("%Y/%m/%d", time());
	if not db.lootDB[today] then
		--db.lootDB = {};
		db.lootDB[today] = {};
	end
	
	if not db.lootDB[today][name] then
		db.lootDB[today][name] = {};
	end
	
	table.insert(db.lootDB[today][name], item);
end

function SFLoot:GetItemDBNumItems(name)
	if not db.lootDB[today] then 
		return 0;
	elseif not db.lootDB[today][name] then
		return 0;
	else
		return #db.lootDB[today][name];
	end
end

function SFLoot:SendCommand(target, command, ...)
	-- send all data as a table, and let receiver unpack it
	local toSend = self:Serialize(command, {...})

	if target == "group" then
		if GetNumGroupMembers() > 0 then -- SendAddonMessage auto converts it to party if needed
			self:SendCommMessage("SFLoot", toSend, "RAID")
		--[[elseif num > 0 then -- Party
			self:SendCommMessage("RCLootCouncil", toSend, "PARTY")]]
		else--if self.testMode then -- Alone (testing)
			self:SendCommMessage("SFLoot", toSend, "WHISPER", self.playerName)
		end

	elseif target == "guild" then
		self:SendCommMessage("SFLoot", toSend, "GUILD")

	else
		if self:UnitIsUnit(target,"player") then -- If target == "player"
			self:SendCommMessage("SFLoot", toSend, "WHISPER", self.playerName)
		else
			-- We cannot send "WHISPER" to a crossrealm player
			if target:find("-") then
				if target:find(self.realmName) then -- Our own realm, just send it
					self:SendCommMessage("SFLoot", toSend, "WHISPER", target)
				else
					return
				end
			else -- Should also be our own realm
				self:SendCommMessage("SFLoot", toSend, "WHISPER", target)
			end
		end
	end
end

function SFLoot:OnCommReceived(prefix, serializedMsg, distri, sender)
	if prefix == "SFLoot" then
		local test, command, data = self:Deserialize(serializedMsg)
		-- NOTE: Since I can't find a better way to do this, all xrealms comms is routed through here
		--			to make sure they get delivered properly. Must be included in every OnCommReceived() function.
		--if self:HandleXRealmComms(self, command, data, sender) then return end
		if test then
			if command == "lootTable" then
				if self:UnitIsUnit(sender, self.masterLooter) and self.enabled then
					lootTable = unpack(data)
					self:SendCommand("group", "lootAck", self.playerName) -- send ack
					self:MainFrameShow();
				else -- a non-ML send a lootTable?!
					
				end
			elseif command == "request_loot" and self.isMasterLooter then
				self:SendCommand(sender, "lootTable", lootTable);
			elseif command == "lootRoll" then
				local roll = unpack(data);
				self:HandleLootRoll(roll)
			elseif command == "awarded" then
				local name, index = unpack(data);
				self:ItemAwarded(name, index);
			elseif command == "session_end" then
				self:Debug("Loot Session ended.")
				self:Print("Alle Items wurden vergeben.")
				--frames.main:Hide();
			elseif command == "change_response" then
				local item, candidateName, response = unpack(data);
				if lootTable[item] and lootTable[item].candidates[candidateName] then
					lootTable[item].candidates[candidateName].response = response;
					if item == currentItem then
						self:SwitchItem(currentItem)
					end
				end
			elseif command == "version" and not self:UnitIsUnit(sender, "player") then
				local s_version = unpack(data);
				if (strfind(s_version, "beta")) then
					self:Debug("Ignoring Beta Version")				
					return
				elseif self.version < s_version and not self.newVersion then
					self:Print("Eine neuere Version ist verfügbar:", s_version)
					self.newVersion = true;
				end
			end
		end
	end
end

function SFLoot:GetML()
	--[[if GetNumGroupMembers() == 0 then -- always the player when testing alone
		return true, self.playerName
	end]]--
	local lootMethod, mlPartyID, mlRaidID = GetLootMethod()
	if lootMethod == "master" then
		local name;
		if mlRaidID then 				-- Someone in raid
			name = self:UnitName("raid"..mlRaidID)
		elseif mlPartyID == 0 then -- Player in party
			name = self.playerName
		elseif mlPartyID then		-- Someone in party
			name = self:UnitName("party"..mlPartyID)
		end
		if name then 
			self.enabled = true 
		else
			self.enabled = false
		end
		
		return IsMasterLooter(), name
	end
	return false, nil;
end

function SFLoot:NewMLCheck()
	self.isMasterLooter, self.masterLooter = self:GetML()
	if self.masterLooter and self.masterLooter ~= "" and strfind(self.masterLooter, "Unknown") then
		self:Debug("Unknown ML")
		return self:ScheduleTimer("NewMLCheck", 2)
	end
end

function SFLoot:OnRaidEnter(arg)
	local instance_type = select(2, GetInstanceInfo())
	if instance_type == "raid" and not self.masterLooter and UnitIsGroupLeader("player") and InGuildParty() then
		SetLootMethod("master", UnitName("player"))
		self:Print(" You are now the Master Looter and SFLoot is now handling looting.")
		SetLootThreshold(4)
		self.isMasterLooter, self.masterLooter = true, self.playerName
	end
end

function SFLoot:SwitchItem(s)
	-- Start with setting up some statics
	local old = currentItem
	currentItem = s
	local t = lootTable[s] -- Shortcut
	frames.main.itemIcon:SetNormalTexture(t.texture)
	frames.main.itemText:SetText(t.link)
	frames.main.iState:SetText(self:GetItemStatus(t.link))
	frames.main.itemLvl:SetText(format("ilvl: %d", t.ilvl))
	-- Set a proper item type text
	if t.subType and t.subType ~= "Miscellaneous" and t.subType ~= "Junk" and t.equipLoc ~= "" then
		frames.main.itemType:SetText(getglobal(t.equipLoc)..", "..t.subType); -- getGlobal to translate from global constant to localized name
	elseif t.subType ~= "Miscellaneous" and t.subType ~= "Junk" then
		frames.main.itemType:SetText(t.subType)
	else
		frames.main.itemType:SetText(getglobal(t.equipLoc));
	end
	
	if t.awarded then
		frames.main.awardString:SetText("Item vergeben an: "..t.awarded);
		frames.main.awardString:SetTextColor(0, 1, 0, 1) 
	else
		frames.main.awardString:SetText("Item nocht nicht vergeben");
		frames.main.awardString:SetTextColor(1, 1, 0, 1) 
	end
	
	if t.candidates[self.playerName] then
		frames.main.mainBtn:Disable();
		frames.main.offBtn:Disable();
		frames.main.transmogBtn:Disable();
		if t.candidates[self.playerName].response == 1 then
			frames.main.passBtn:Disable();
		else
			frames.main.passBtn:Enable();
		end
	else
		frames.main.mainBtn:Enable();
		frames.main.offBtn:Enable();
		frames.main.transmogBtn:Enable();
		frames.main.passBtn:Disable();
	end
	
	-- Update the session buttons
	frames.itemButtons[s] = self:UpdateItemButton(s, t.texture, t.link, t.awarded)
	frames.itemButtons[old] = self:UpdateItemButton(old, lootTable[old].texture, lootTable[old].link, lootTable[old].awarded)

	-- Since we switched sessions, we want to sort by response
	self:BuildST()
	frames.main.st.cols[3].sort = "desc"
	frames.main.st:SortData();
end

function SFLoot:MainFrameShow()
	self:CreateRollFrame();
	self:CreateItemButtons();
	self:BuildST();
	frames.main:Show();
end

function SFLoot:CreateItemButtons()
	if #lootTable == 0 then return end
	
	local item;
	
	for i = 1, #lootTable do
		item = lootTable[i];
		frames.itemButtons[i] = self:UpdateItemButton(i, item.texture, item.link, item.awarded)
	end
	self:SwitchItem(1);
end

function SFLoot:UpdateItemButton(i, texture, link, awarded)
	local sessionButtons = frames.itemButtons;
	local btn = sessionButtons[i]
	if not btn then -- create the button
		btn = CreateFrame("Button", "SFButton"..i, frames.main)
		btn:SetSize(40,40)
		--btn:SetText(i)
		if i == 1 then
			btn:SetPoint("TOPRIGHT", frames.main, "TOPLEFT")
		else
			btn:SetPoint("TOP", sessionButtons[i-1], "BOTTOM", 0, -2)
		end
		btn:SetScript("Onclick", function() SFLoot:SwitchItem(i); end)
		btn:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square")
		btn:GetHighlightTexture():SetBlendMode("ADD")
		btn:SetNormalTexture(texture or "Interface\\InventoryItems\\WoWUnknownItem01")
		btn:GetNormalTexture():SetDrawLayer("BACKGROUND")
	end
	-- then update it
	btn:SetNormalTexture(texture or "Interface\\InventoryItems\\WoWUnknownItem01")
	-- Set the colored border and tooltips
	btn:SetBackdrop({
		bgFile = "",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		edgeSize = 18,
		--insets = { left = -4, right = -4, top = -4, bottom = -4 }
	})
	local lines = { format("Click to switch to %s", link or "") }
	if i == currentItem then
		btn:SetBackdropBorderColor(1,1,0,1) -- yellow
		--btn:SetBackdropColor(1,1,1,1)
		btn:GetNormalTexture():SetVertexColor(1,1,1)
	elseif awarded then
		btn:SetBackdropBorderColor(0,1,0,1) -- green
		--btn:SetBackdropColor(1,1,1,0.8)
		btn:GetNormalTexture():SetVertexColor(0.8,0.8,0.8)
		tinsert(lines, "Item wurde vergeben!")
	else
		btn:SetBackdropBorderColor(1,1,1,1) -- white
		--btn:SetBackdropColor(0.5,0.5,0.5,0.8)
		btn:GetNormalTexture():SetVertexColor(0.5,0.5,0.5)
	end
	--btn:SetScript("OnEnter", function() SFLoot:CreateTooltip(unpack(lines)) end)
	--btn:SetScript("OnLeave", function() SFLoot:HideTooltip() end)
	btn:SetScript("OnEnter", function() SFLoot:CreateHypertip(link) end)
	btn:SetScript("OnLeave", function() SFLoot:HideTooltip() end)
	
	return btn;
end

function SFLoot:CreateRollFrame()
	if frames.main then return end
	
	local f = self:CreateFrame("SFLootFrame", "mainframe", "SFLootMaster", 400, 400);
	
	local st = LibStub("ScrollingTable"):CreateST(self.scrollCols, NUM_ROWS, ROW_HEIGHT, { ["r"] = 1.0, ["g"] = 0.9, ["b"] = 0.0, ["a"] = 0.5 }, f.content);
	st.frame:SetPoint("BOTTOMLEFT", f, "BOTTOMLEFT", 10, 10);
	st:RegisterEvents({
		["OnClick"] = function(rowFrame, cellFrame, data, cols, row, realrow, column, table, button, ...)
			if button == "RightButton" and row then
				if self.isMasterLooter then
					menuFrame.name = data[realrow].name
					Lib_ToggleDropDownMenu(1, nil, menuFrame, cellFrame, 0, 0);
				end
			end
			-- Return false to have the default OnClick handler take care of left clicks
			return false
		end,
	})	
	st:EnableSelection(true)
	f.st = st
	
	local item = CreateFrame("Button", nil, f.content)
	item:EnableMouse()
    item:SetNormalTexture("Interface/ICONS/INV_Misc_QuestionMark")
    item:SetScript("OnEnter", function()
		if not lootTable then return; end
		SFLoot:CreateHypertip(lootTable[currentItem].link)
	end)
	item:SetScript("OnLeave", SFLoot.HideTooltip)
	item:SetScript("OnClick", function()
		if not lootTable then return; end
	    if ( IsModifiedClick() ) then
		    HandleModifiedItemClick(lootTable[currentItem].link);
        end
    end);
	item:SetPoint("TOPLEFT", f, "TOPLEFT", 10, -20)
	item:SetSize(50,50)
	f.itemIcon = item

	local iTxt = f.content:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge")
	iTxt:SetPoint("TOPLEFT", item, "TOPRIGHT", 10, 0)
	iTxt:SetText("Fehler") -- Set text for reasons
	f.itemText = iTxt

	local ilvl = f.content:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	ilvl:SetPoint("TOPLEFT", iTxt, "BOTTOMLEFT", 0, -4)
	ilvl:SetTextColor(1, 1, 1) -- White
	ilvl:SetText("")
	f.itemLvl = ilvl

	local iState = f.content:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	iState:SetPoint("LEFT", ilvl, "RIGHT", 5, 0)
	iState:SetTextColor(0,1,0,1) -- Green
	iState:SetText("")
	f.iState = iState

	local iType = f.content:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	iType:SetPoint("TOPLEFT", ilvl, "BOTTOMLEFT", 0, -4)
	iType:SetTextColor(0.5, 1, 1) -- Turqouise
	iType:SetText("")
	f.itemType = iType

	local awdstr = f.content:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	awdstr:SetPoint("TOPLEFT", f.content, "TOPLEFT", 15, -80)
	awdstr:SetText("Item noch nicht vergeben")
	awdstr:SetTextColor(1, 1, 0, 1) -- Yellow

	f.awardString = awdstr	
	
	local b = CreateFrame("Button", nil, f.content, "UIPanelButtonTemplate")
	b:SetText("Main")
	b:SetSize(80,25)
	b:SetPoint("TOPLEFT", f, "TOPLEFT", 10, -110)
	b:SetScript("OnClick", function()
		SFLoot:LootRoll(self.playerName, 4);
	end)
	f.mainBtn = b	

	local b2 = CreateFrame("Button", nil, f.content, "UIPanelButtonTemplate")
	b2:SetText("Off")
	b2:SetSize(80,25)
	b2:SetPoint("TOPLEFT", b, "TOPLEFT", 90, 0)
	b2:SetScript("OnClick", function()
		SFLoot:LootRoll(self.playerName, 3);
	end)
	f.offBtn = b2

	local b3 = CreateFrame("Button", nil, f.content, "UIPanelButtonTemplate")
	b3:SetText("Transmog")
	b3:SetSize(80,25)
	b3:SetPoint("TOPLEFT", b2, "TOPLEFT", 90, 0)
	b3:SetScript("OnClick", function()
		SFLoot:LootRoll(self.playerName, 2);
	end)
	f.transmogBtn = b3	

	local b4 = CreateFrame("Button", nil, f.content, "UIPanelButtonTemplate")
	b4:SetText("Pass")
	b4:SetSize(80,25)
	b4:SetPoint("TOPLEFT", b3, "TOPLEFT", 90, 0)
	b4:SetScript("OnClick", function()
		SFLoot:LootRoll(self.playerName, 1);
	end)
	f.passBtn = b4	
	
	local close = CreateFrame("Button", nil, f.content, "UIPanelCloseButton")
	close:SetPoint("TOPRIGHT", 2, 1)
	close:SetScript("OnClick", function() f:Hide() end);
	
	f.closeBtn = close;
	
	f:SetWidth(st.frame:GetWidth() + 20)
	
	frames.main = f;
end

function SFLoot:BuildST()
	local rows = {}
	local i = 1
	for name in pairs(lootTable[currentItem].candidates) do
		rows[i] = {
			name = name,
			cols = {
				{ value = "",	DoCellUpdate = self.SetCellClass,		name = "class",},
				{ value = "",	DoCellUpdate = self.SetCellName,			name = "name",},
				{ value = "",	DoCellUpdate = self.SetCellResponse,	name = "response",},
				{ value = "",	DoCellUpdate = self.SetCellIlvl,			name = "ilvl",},
				{ value = "",	DoCellUpdate = self.SetCellDiff,			name = "items",},
				{ value = "",	DoCellUpdate = self.SetCellGear, 		name = "gear1",},
				{ value = "",	DoCellUpdate = self.SetCellGear, 		name = "gear2",},
				{ value = "",	DoCellUpdate = self.SetCellItems,			name = "items",},
				{ value = "",	DoCellUpdate = self.SetCellRoll,			name = "roll"},
			},
		}
		i = i + 1
	end
	frames.main.st:SetData(rows)
end

function SFLoot:CreateFrame(name, cName, title, width, height)
	local f = CreateFrame("Frame", name, nil) -- LibWindow seems to work better with nil parent
	f:Hide()
	f:SetFrameStrata("DIALOG")
	f:SetWidth(450)
	f:SetHeight(height or 325)
	lwin:Embed(f)
	f:RegisterConfig(db.UI[cName])
	f:RestorePosition() -- might need to move this to after whereever GetFrame() is called
	f:MakeDraggable()
	f:SetScript("OnMouseWheel", function(f,delta) if IsControlKeyDown() then lwin.OnMouseWheel(f,delta) end end)

	local tf = CreateFrame("Frame", nil, f)
	--tf:SetFrameStrata("DIALOG")
	tf:SetToplevel(true)
	tf:SetBackdrop({
	     bgFile = db.UI.default.background,
	     edgeFile = db.UI.default.border,
	     tile = true, tileSize = 64, edgeSize = 12,
	     insets = { left = 2, right = 2, top = 2, bottom = 2 }
	})
	tf:SetBackdropColor(unpack(db.UI.default.bgColor))
	tf:SetBackdropBorderColor(unpack(db.UI.default.borderColor))
	tf:SetHeight(22)
	tf:EnableMouse()
	tf:SetMovable(true)
	tf:SetWidth((width/2) or 250)
	tf:SetPoint("CENTER",f,"TOP",0,-1)
	tf:SetScript("OnMouseDown", function(self) self:GetParent():StartMoving() end)
	tf:SetScript("OnMouseUp", function(self) -- Get double click by trapping time betweem mouse up
		local frame = self:GetParent()
		frame:StopMovingOrSizing()
		frame:SavePosition()
		if self.lastClick and GetTime() - self.lastClick <= 0.5 then
			self.lastClick = nil
			if frame.minimized then frame:Maximize() else frame:Minimize() end
		else
			self.lastClick = GetTime()
		end
	end)

	local text = tf:CreateFontString(nil,"OVERLAY","GameFontNormal")
	text:SetPoint("CENTER",tf,"CENTER")
	text:SetTextColor(1,1,1,1)
	text:SetText(title)
	tf.text = text
	f.title = tf

	local c = CreateFrame("Frame", nil, f) -- frame that contains the actual content
	c:SetBackdrop({
	bgFile = db.UI.default.background,
		edgeFile = db.UI.default.border,	   tile = true, tileSize = 64, edgeSize = 12,
		insets = { left = 2, right = 2, top = 2, bottom = 2 }
	})
	c:EnableMouse(true)
	c:SetWidth(450)
	c:SetHeight(height or 325)
	c:SetBackdropColor(unpack(db.UI.default.bgColor))
	c:SetBackdropBorderColor(unpack(db.UI.default.borderColor))
	c:SetPoint("TOPLEFT")
	c:SetScript("OnMouseDown", function(self) self:GetParent():StartMoving() end)
	c:SetScript("OnMouseUp", function(self) self:GetParent():StopMovingOrSizing(); self:GetParent():SavePosition() end)
	f.content = c
	f.minimized = false
	f.IsMinimized = function(frame) return frame.minimized end
	f.Minimize = function(frame)
		--self:Debug("Minimize()")
		if not frame.minimized then
		  	frame.content:Hide()
			frame.minimized = true
		end
	end
	f.Maximize = function(frame)
		--self:Debug("Maximize()")
		if frame.minimized then
		  	frame.content:Show()
			frame.minimized = false
		end
	end
	-- Support for auto hide in combat:
	tinsert(frames, f)
	local old_setwidth = f.SetWidth
	f.SetWidth = function(self, width) -- Hack so we only have to set width once
		old_setwidth(self, width)
		self.content:SetWidth(width)
	end
	local old_setheight = f.SetHeight
	f.SetHeight = function(self, height)
		old_setheight(self, width)
		self.content:SetHeight(height)
	end

	return f
end

function SFLoot:GetDiffColor(num)
	if num == "" then num = 0 end -- Can't compare empty string
	local green, red, grey = {0,1,0,1},{1,0,0,1},{0.75,0.75,0.75,1}
	if num > 0 then return green end
	if num < 0 then return red end
	return grey
end

function SFLoot.SetCellClass(rowFrame, frame, data, cols, row, realrow, column, fShow, table, ...)
	local name = data[realrow].name
	SFLoot.SetCellClassIcon(rowFrame, frame, data, cols, row, realrow, column, fShow, table, lootTable[currentItem].candidates[name].class)
end

function SFLoot.SetCellName(rowFrame, frame, data, cols, row, realrow, column, fShow, table, ...)
	local name = data[realrow].name
	frame.text:SetText(SFLoot.Ambiguate(name))
	local c = SFLoot:GetClassColor(lootTable[currentItem].candidates[name].class)
	frame.text:SetTextColor(c.r, c.g, c.b, c.a)
	data[realrow].cols[column].value = name
end

function SFLoot.SetCellResponse(rowFrame, frame, data, cols, row, realrow, column, fShow, table, ...)
	local name = data[realrow].name
	frame.text:SetText(responses[lootTable[currentItem].candidates[name].response])
	--frame.text:SetTextColor(SFLoot:GetResponseColor(lootTable[currentItem].candidates[name].response))
end

function SFLoot.SetCellIlvl(rowFrame, frame, data, cols, row, realrow, column, fShow, table, ...)
	local name = data[realrow].name
	frame.text:SetText(SFLoot.round(lootTable[currentItem].candidates[name].ilvl), 2)
	data[realrow].cols[column].value = lootTable[currentItem].candidates[name].ilvl
end

function SFLoot.SetCellDiff(rowFrame, frame, data, cols, row, realrow, column, fShow, table, ...)
	local name = data[realrow].name
	frame.text:SetText(lootTable[currentItem].candidates[name].diff);
	frame.text:SetTextColor(unpack(SFLoot:GetDiffColor(lootTable[currentItem].candidates[name].diff)))
	data[realrow].cols[column].value = lootTable[currentItem].candidates[name].diff
end

function SFLoot.SetCellGear(rowFrame, frame, data, cols, row, realrow, column, fShow, table, ...)
	local gear = data[realrow].cols[column].name -- gear1 or gear2
	local name = data[realrow].name
	gear = lootTable[currentItem].candidates[name][gear] -- Get the actual gear
	if gear then
		local texture = select(10, GetItemInfo(gear))
		frame:SetNormalTexture(texture)
		frame:SetScript("OnEnter", function() SFLoot:CreateHypertip(gear) end)
		frame:SetScript("OnLeave", function() SFLoot:HideTooltip() end)
		frame:Show()
	else
		frame:Hide()
	end
end

function SFLoot.SetCellItems(rowFrame, frame, data, cols, row, realrow, column, fShow, table, ...)
	local name = data[realrow].name
	frame.text:SetText(SFLoot:GetItemDBNumItems(name));
	data[realrow].cols[column].value = SFLoot:GetItemDBNumItems(name)
end

function SFLoot.SetCellRoll(rowFrame, frame, data, cols, row, realrow, column, fShow, table, ...)
	local name = data[realrow].name
	frame.text:SetText(lootTable[currentItem].candidates[name].roll)
	data[realrow].cols[column].value = lootTable[currentItem].candidates[name].roll
end

function SFLoot.SetCellClassIcon(rowFrame, frame, data, cols, row, realrow, column, fShow, table, class)
	local celldata = data[realrow].cols and data[realrow].cols[column] or data[realrow][column]
	local class = celldata.args and celldata.args[1] or class
	if class then
		frame:SetNormalTexture("Interface\\GLUES\\CHARACTERCREATE\\UI-CHARACTERCREATE-CLASSES"); -- this is the image containing all class icons
		local coords = CLASS_ICON_TCOORDS[class]; -- get the coordinates of the class icon we want
		frame:GetNormalTexture():SetTexCoord(unpack(coords)); -- cut out the region with our class icon according to coords
	else -- if there's no class
		frame:SetNormalTexture("Interface/ICONS/INV_Sigil_Thorim.png")
	end
end

--- Returns a color table for use with lib-st
function SFLoot:GetClassColor(class)
	local color = RAID_CLASS_COLORS[class]
	if not color then
		-- if class not found, return epic color.
		return {r=1,g=1,b=1,a=1}--{["r"] = 0.63921568627451, ["g"] = 0.2078431372549, ["b"] = 0.93333333333333, ["a"] = 1.0 };
	else
		color.a = 1.0
		return color
	end
end

function ResponseSort(table, rowa, rowb, sortbycol)
	local column = table.cols[sortbycol]
	local a, b = table:GetRow(rowa), table:GetRow(rowb);
	a, b = lootTable[currentItem].candidates[a.name].response, lootTable[currentItem].candidates[b.name].response
	if a == b then
		if column.sortnext then
			local nextcol = table.cols[column.sortnext];
			if not(nextcol.sort) then
				if nextcol.comparesort then
					return nextcol.comparesort(table, rowa, rowb, column.sortnext);
				else
					return table:CompareSort(rowa, rowb, column.sortnext);
				end
			end
		end
		return false
	else
		return a > b;
	end
end

function RollSort(table, rowa, rowb, sortbycol)
	local column = table.cols[sortbycol]
	local a, b = table:GetRow(rowa), table:GetRow(rowb);
	a, b = lootTable[currentItem].candidates[a.name].roll, lootTable[currentItem].candidates[b.name].roll
	if a == b then
		return false
	else
		return a > b;
	end
end

function SFLoot:UnitIsUnit(unit1, unit2)
	if not unit1 or not unit2 then return false end
	-- Remove realm names, if any
	if strfind(unit1, "-", nil, true) ~= nil then
		unit1 = Ambiguate(unit1, "short")
	end
	if strfind(unit2, "-", nil, true) ~= nil then
		unit2 = Ambiguate(unit2, "short")
	end
	return UnitIsUnit(unit1, unit2)
end

-- We always want realm name when we call UnitName
-- Note: If 'unit' is a playername, that player must be in our raid or party!
--[[ NOTE I'm concerned about the UnitIsVisible() range thing with UnitName(),
 	although testing in party doesn't seem to affect it. To counter this, it'll
	return any "unit" with something after a "-" (i.e a name from GetRaidRosterInfo())
	which means it isn't useable with all the "name-target" unitIDs]]
function SFLoot:UnitName(unit)
	-- First strip any spaces
	unit = gsub(unit, " ", "")
	-- Then see if we already have a realm name appended
	local find = strfind(unit, "-", nil, true)
	if find and find < #unit then -- "-" isn't the last character
		-- Let's give it same treatment as below so we're sure it's the same
		local name, realm = strsplit("-", unit, 2)
		name = name:lower():gsub("^%l", string.upper)
		return name.."-"..realm
	end
	-- Apparently functions like GetRaidRosterInfo() will return "real" name, while UnitName() won't
	-- always work with that (see ticket #145). We need this to be consistant, so just lowercase the unit:
	unit = unit:lower()
	-- Proceed with UnitName()
	local name, realm = UnitName(unit)
	if not realm or realm == "" then realm = self.realmName end -- Extract our own realm
	-- We also want to make sure the returned name is always title cased (it might not always be! ty Blizzard)
	name = name:lower():gsub("^%l", string.upper)
	return name and name.."-"..realm or nil
end

--- Displays a tooltip anchored to the mouse
-- @paramsig ...
-- @param ... Lines to be added.
function SFLoot:CreateTooltip(...)
	GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR")
	for i = 1, select("#", ...) do
		GameTooltip:AddLine(select(i, ...),1,1,1)
	end
	GameTooltip:Show()
end

--- Displays a hyperlink tooltip
-- @paramsig link
-- @param link The link to display
function SFLoot:CreateHypertip(link)
	if not link or link == "" then return end
	GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR")
	GameTooltip:SetHyperlink(link)
end

--- Hide the tooltip created with :CreateTooltip()
function SFLoot:HideTooltip()
	GameTooltip:Hide()
end

function SFLoot:GetItemStatus(item)
	-- addon:Debug("GetitemStatus", item)
	if not item then return "" end
	GameTooltip:SetOwner(UIParent, "ANCHOR_NONE")
	GameTooltip:SetHyperlink(item)
	local text = ""
	if GameTooltip:NumLines() > 1 then -- check that there is something here
		local line = getglobal('GameTooltipTextLeft2') -- Should always be line 2
		local t =  line:GetText()
		-- The following color string should be there if we have a green status text
		if t then
			if strfind(t, "cFF 0FF 0") then
				text = t
			end
		end
	end
	GameTooltip:Hide()
	return text
end

local INVTYPE_Slots = {
		INVTYPE_HEAD		    = "HeadSlot",
		INVTYPE_NECK		    = "NeckSlot",
		INVTYPE_SHOULDER	    = "ShoulderSlot",
		INVTYPE_CLOAK		    = "BackSlot",
		INVTYPE_CHEST		    = "ChestSlot",
		INVTYPE_WRIST		    = "WristSlot",
		INVTYPE_HAND		    = "HandsSlot",
		INVTYPE_WAIST		    = "WaistSlot",
		INVTYPE_LEGS		    = "LegsSlot",
		INVTYPE_FEET		    = "FeetSlot",
		INVTYPE_SHIELD		    = "SecondaryHandSlot",
		INVTYPE_ROBE		    = "ChestSlot",
		INVTYPE_2HWEAPON	    = {"MainHandSlot","SecondaryHandSlot"},
		INVTYPE_WEAPONMAINHAND	= "MainHandSlot",
		INVTYPE_WEAPONOFFHAND	= {"SecondaryHandSlot",["or"] = "MainHandSlot"},
		INVTYPE_WEAPON		    = {"MainHandSlot","SecondaryHandSlot"},
		INVTYPE_THROWN		    = {"SecondaryHandSlot", ["or"] = "MainHandSlot"},
		INVTYPE_RANGED		    = {"SecondaryHandSlot", ["or"] = "MainHandSlot"},
		INVTYPE_RANGEDRIGHT 	= {"SecondaryHandSlot", ["or"] = "MainHandSlot"},
		INVTYPE_FINGER		    = {"Finger0Slot","Finger1Slot"},
		INVTYPE_HOLDABLE	    = {"SecondaryHandSlot", ["or"] = "MainHandSlot"},
		INVTYPE_TRINKET		    = {"TRINKET0SLOT", "TRINKET1SLOT"}
}

local SFTokenTable = {
	
}

function SFLoot:GetPlayersGear(link, equipLoc)
	local itemID = self:GetItemIDFromLink(link) -- Convert to itemID
	--self:DebugLog("GetPlayersGear", itemID, equipLoc)
	if not itemID then return nil, nil; end
	local item1, item2;
	-- check if the item is a token, and if it is, return the matching current gear
	if SFTokenTable[itemID] then
		if SDTokenTable[itemID] == "Trinket" then -- We need to return both trinkets
			item1 = GetInventoryItemLink("player", GetInventorySlotInfo("TRINKET0SLOT"))
			item2 = GetInventoryItemLink("player", GetInventorySlotInfo("TRINKET1SLOT"))
		else	-- Just return the slot from the tokentable
			item1 = GetInventoryItemLink("player", GetInventorySlotInfo(SFTokenTable[itemID]))
		end
		return item1, item2
	end
	local slot = INVTYPE_Slots[equipLoc]
	if not slot then return nil, nil; end;
	item1 = GetInventoryItemLink("player", GetInventorySlotInfo(slot[1] or slot))
	if not item1 and slot['or'] then
		item1 = GetInventoryItemLink("player", GetInventorySlotInfo(slot['or']))
	end;
	if slot[2] then
		item2 = GetInventoryItemLink("player", GetInventorySlotInfo(slot[2]))
	end
	return item1, item2;
end

function SFLoot:GetArtifactRelics(link)
	local id = self:GetItemIDFromLink(link)
	local g1,g2;
	for i = 1, C_ArtifactUI.GetEquippedArtifactNumRelicSlots() do
		if C_ArtifactUI.CanApplyRelicItemIDToEquippedArtifactSlot(id,i) then -- We can equip it
			if g1 then
				g2 = select(4,C_ArtifactUI.GetEquippedArtifactRelicInfo(i))
			else
				g1 = select(4,C_ArtifactUI.GetEquippedArtifactRelicInfo(i))
			end
		end
	end
	return g1, g2
end

-- Used to find localized subType names
local subTypeLookup = {
	["Cloth"]					= 124168, -- Felgrease-Smudged Robes
	["Leather"] 				= 124265, -- Leggings of Eternal Terror
	["Mail"] 					= 124291, -- Eredar Fel-Chain Gloves
	["Plate"]					= 124322, -- Treads of the Defiler
	["Shields"] 				= 124354, -- Felforged Aegis
	["Bows"] 					= 128194, -- Snarlwood Recurve Bow
	["Crossbows"] 				= 124362, -- Felcrystal Impaler
	["Daggers"]					= 124367, -- Fang of the Pit
	["Guns"]						= 124370, -- Felfire Munitions Launcher
	["Fist Weapons"] 			= 124368, -- Demonblade Eviscerator
	["One-Handed Axes"]		= 128196, -- Limbcarver Hatchet
	["One-Handed Maces"]		= 124372, -- Gavel of the Eredar
	["One-Handed Swords"] 	= 124387, -- Shadowrend Talonblade
	["Polearms"] 				= 124377, -- Rune Infused Spear
	["Staves"]					= 124382, -- Edict of Argus
	["Two-Handed Axes"]		= 124360, -- Hellrender
	["Two-Handed Maces"]		= 124375, -- Maul of Tyranny
	["Two-Handed Swords"]	= 124389, -- Calamity's Edge
	["Wands"]					= 128096, -- Demonspine Wand
	["Warglaives"]				= 141604, -- Glaive of the Fallen
	["Artifact Relic"]		= 141271, -- Hope of the Forest
}

function SFLoot:LocalizeSubTypes()
	if self.db.global.localizedSubTypes.created then return end -- We only need to create it once
	-- Get the item info
	for _, item in pairs(subTypeLookup) do
		GetItemInfo(item)
	end
	self.db.global.localizedSubTypes = {} -- reset
	self.db.global.localizedSubTypes.created = true
	for name, item in pairs(subTypeLookup) do
		local sType = select(7, GetItemInfo(item))
		if sType then
			self.db.global.localizedSubTypes[sType] = name
			self:Debug("Found "..name.." localized as: "..sType)
		else -- Probably not cached, set a timer
			self:Debug("We didn't find:", name, item)
			self:ScheduleTimer("LocalizeSubTypes", 2)
			self.db.global.localizedSubTypes.created = false
			return
		end
	end
end

function SFLoot:IsItemBoE(item)
	if not item then return false end
	GameTooltip:SetOwner(UIParent, "ANCHOR_NONE")
	GameTooltip:SetHyperlink(item)
	if GameTooltip:NumLines() > 1 then -- check that there is something here
		for i = 1, 5 do -- BoE status won't be further away than line 5
			local line = getglobal('GameTooltipTextLeft' .. i)
			if line and line.GetText then
				if line:GetText() == ITEM_BIND_ON_EQUIP then
					GameTooltip:Hide()
					return true
				end
			end
		end
	end
	GameTooltip:Hide()
	return false
end

function SFLoot:GetItemIDFromLink(link)
	return tonumber(strmatch(link, "item:(%d+):"))
end

function SFLoot:GetItemStringFromLink(link)
	return strmatch(link, "item:([%d:]+)")
end

function SFLoot:GetItemNameFromLink(link)
	return strmatch(link, "%[(.+)%]")
end

function SFLoot.round(num, decimals)
	if type(num) ~= "number" then return "" end
	return tonumber(string.format("%." .. (decimals or 0) .. "f", num))
end

function SFLoot.Ambiguate(name)
	return db.ambiguate and Ambiguate(name, "none") or Ambiguate(name, "short")
end

function SFLoot:Debug(msg, ...)
	if self.debug then
		if select("#", ...) > 0 then
			self:Print("|cffcb6700debug:|r "..tostring(msg).."|cffff6767", ...)
		else
			self:Print("|cffcb6700debug:|r "..tostring(msg).."|r")
		end
	end
end

do
	local info -- Efficiency :)
	-- NOTE Take care of info[] values when inserting new buttons
	function SFLoot.RightClickMenu(menu, level)
		if not SFLoot.isMasterLooter then return end

		local candidateName = menu.name
		--local data = lootTable[currentItem].candidates[candidateName] -- Shorthand

		if level == 1 then
			info = Lib_UIDropDownMenu_CreateInfo()
			info.text = SFLoot.Ambiguate(candidateName)
			info.isTitle = true
			info.notCheckable = true
			info.disabled = true
			Lib_UIDropDownMenu_AddButton(info, level)

			info.text = ""
			info.isTitle = false
			Lib_UIDropDownMenu_AddButton(info, level)

			info.text = "Item zuweisen"
			info.func = function()
				SFLoot:AwardItem(candidateName, currentItem)
			end
			info.disabled = false
			Lib_UIDropDownMenu_AddButton(info, level)

			info.text = ""
			info.isTitle = false
			Lib_UIDropDownMenu_AddButton(info, level)

			info.text = "Verwendung ändern"
			info.value = "CHANGE_RESPONSE"
			info.hasArrow = true
			info.disabled = false
			Lib_UIDropDownMenu_AddButton(info, level)			
		elseif level == 2 then
			local value = LIB_UIDROPDOWNMENU_MENU_VALUE
			info = Lib_UIDropDownMenu_CreateInfo()

			if value == "CHANGE_RESPONSE" then
				for i = 1, #responses do
					local v = responses[i]
					info.text = v
					info.notCheckable = true
					info.func = function()
							SFLoot:SendCommand("group", "change_response", currentItem, candidateName, i)
					end
					Lib_UIDropDownMenu_AddButton(info, level)
				end
			end
		end
	end
end
